/////////////////////////////////////////////////////////////////////////////
//
// wxWebServer - a simple web server using wxWidgets framework
//
// File:        contenttypes.cpp
// 
// Author:      Fedja Stevanovic
// 
// Copyright:   Copyright (C) 2007 Fedja Stevanovic (tutubibi.net)
//
// Licence:
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////////////

#if defined(__GNUG__) && !defined(NO_GCC_PRAGMA)
#pragma implementation "contenttypes.h"
#endif

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include "contenttypes.h"

bool ContentTypes::exists = false;
ContentTypes* ContentTypes::instance = NULL;


/*!
 * ContentTypes constructors
 */

ContentTypes::ContentTypes( )
{
	contentTypes[_("txt")] = _("text/plain");
	contentTypes[_("html")] = _("text/html");
	contentTypes[_("htm")] = _("text/html");
	contentTypes[_("css")] = _("text/css");
	contentTypes[_("js")] = _("text/javascript");
	contentTypes[_("xml")] = _("text/xml");

	contentTypes[_("gif")] = _("image/gif");
	contentTypes[_("jpg")] = _("image/jpeg");
	contentTypes[_("png")] = _("image/png");
	
	contentTypes[_("zip")] = _("application/zip");
	contentTypes[_("pdf")] = _("application/pdf");
	contentTypes[_("swf")] = _("application/x-shockwave-flash");

	contentTypes[_("mp3")] = _("audio/mpeg");
	contentTypes[_("wav")] = _("audio/wav");
	contentTypes[_("wma")] = _("audio/x-ms-wma");
	contentTypes[_("aac")] = _("audio/aac");

	contentTypes[_("qt")] = _("video/quicktime");
	contentTypes[_("mov")] = _("video/quicktime");
	contentTypes[_("3gp")] = _("video/3gpp");
	contentTypes[_("avi")] = _("video/avi");
	contentTypes[_("wmv")] = _("video/x-ms-wmv");

	contentTypes[_("wml")] = _("text/vnd.wap.wml");
	contentTypes[_("jad")] = _("text/vnd.sun.j2me.app-descriptor");
	contentTypes[_("jar")] = _("application/java-archive");
}


/*!
 * ContentTypes destructor
 */

ContentTypes::~ContentTypes()
{
	exists = false;
}

/*!
 * ContentTypes getInstance
 */
ContentTypes* ContentTypes::getInstance()
{
    if( !exists)
    {
        instance = new ContentTypes();
        exists = true;
        return instance;
    }
    else
    {
        return instance;
    }
}

/*!
 * ContentTypes getContentType
 */
wxString ContentTypes::getContentType( const wxString& fileExt )
{
	wxString value = wxEmptyString;
	ContentTypesHash::iterator it;
	it = contentTypes.find( fileExt );
	if( it != contentTypes.end() )
        value = it->second;

	return value;
}

/*!
 * ContentTypes getFallbackContentType - if not found, return as this
 */
wxString ContentTypes::getFallbackContentType()
{
	return _("application/octet-stream");
}	


