/////////////////////////////////////////////////////////////////////////////
//
// wxWebServer - a simple web server using wxWidgets framework
//
// File:        requesthandler.h
// 
// Author:      Fedja Stevanovic
// 
// Copyright:   Copyright (C) 2007 Fedja Stevanovic (tutubibi.net)
//
// Licence:
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////////////

#ifndef _REQUESTHANDLER_H_
#define _REQUESTHANDLER_H_

#if defined(__GNUG__) && !defined(NO_GCC_PRAGMA)
#pragma interface "requesthandler.h"
#endif

#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif

#include "wx/socket.h"

// has map for headers
WX_DECLARE_STRING_HASH_MAP( wxString, HeadersHash );

/*!
 * RequestHandler class declaration
 */

class CRequestHandler : public wxThread  
{

public:

    CRequestHandler(wxSocketBase* Socket, const wxString& wwwRoot, const wxString& defaultFile, 
  						    const wxString& cgiExt, const wxString& cgiCmd, const wxString& assumedContentType,
							wxString* reqLog, wxString* resLog );
    virtual ~CRequestHandler();
    
    virtual void* Entry();

protected:
	wxString	  m_wwwRoot;
	wxString	  m_defaultFile;
    wxSocketBase* m_Socket;
    wxString      m_cgiExt;
    wxString      m_cgiHandler;
	wxString	  m_assumedContentType;

	wxString	m_localPort;
	wxString	m_localHost;
	wxString	m_peerPort;
	wxString	m_peerHost;

	wxString	m_reqUrl;
	wxString	m_reqFullUrl;
	wxString	m_fullPath;
	wxString	m_reqMethod;
	wxString	m_reqVersion;
	wxString	m_queryString;
	wxString	m_reqFullHeaders;
	HeadersHash	m_reqHeaders;
	short		m_methodCode;

	char*		  m_reqBody;	
	unsigned long m_reqContentLength;

	// for logging in a main window
	wxString*	m_reqLog;
	wxString*	m_resLog;
	
private:
	void* ReturnError(int code, char* description);

	void* DoCGI();

	bool  ParseHeaders();

	char* buf; // where we receive data
   
};

#endif
// _REQUESTHANDLER_H_

