/////////////////////////////////////////////////////////////////////////////
//
// wxWebServer - a simple web server using wxWidgets framework
//
// File:        webservermain.cpp
// 
// Author:      Fedja Stevanovic
// 
// Copyright:   Copyright (C) 2007 Fedja Stevanovic (tutubibi.net)
//
// Licence:
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////////////

#if defined(__GNUG__) && !defined(NO_GCC_PRAGMA)
#pragma implementation "webservermain.h"
#endif

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

////@begin includes
#include "aboutdialog.h"
////@end includes

#include "wx/socket.h"
#include <wx/config.h>
#include "webservermain.h"
#include "requesthandler.h"

////@begin XPM images
#include "wxWebServer.xpm"
////@end XPM images

// maximum size of log output per request
#define MAX_LOG 4096

/*!
 * WebServerMain type definition
 */

IMPLEMENT_DYNAMIC_CLASS( WebServerMain, wxDialog )


/*!
 * WebServerMain event table definition
 */

BEGIN_EVENT_TABLE( WebServerMain, wxDialog )

////@begin WebServerMain event table entries
    EVT_BUTTON( ID_BUTTON_ABOUT, WebServerMain::OnButtonAboutClick )

    EVT_BUTTON( ID_BUTTON_START, WebServerMain::OnButtonStartClick )

    EVT_BUTTON( ID_BUTTON_STOP, WebServerMain::OnButtonStopClick )

    EVT_BUTTON( ID_BUTTON_EXIT, WebServerMain::OnButtonExitClick )

////@end WebServerMain event table entries

	  EVT_SOCKET( ID_SERVER, WebServerMain::OnServerEvent)

END_EVENT_TABLE()


/*!
 * WebServerMain constructors
 */

WebServerMain::WebServerMain()
{
    Init();
}

WebServerMain::WebServerMain( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
    Init();
    Create(parent, id, caption, pos, size, style);
}


/*!
 * WebServerMain creator
 */

bool WebServerMain::Create( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
////@begin WebServerMain creation
    SetExtraStyle(wxWS_EX_BLOCK_EVENTS);
    wxDialog::Create( parent, id, caption, pos, size, style );

    CreateControls();
    SetIcon(GetIconResource(wxT("wxWebServer.xpm")));
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
////@end WebServerMain creation
    return true;
}


/*!
 * WebServerMain destructor
 */

WebServerMain::~WebServerMain()
{
////@begin WebServerMain destruction
////@end WebServerMain destruction

	wxConfig *config = new wxConfig(_("wxWebServer"));
	config->Write(_("wxWebServerDocRoot"), m_DocRoot->GetPath());
	config->Write(_("wxWebServerPort"), m_Port->GetValue());
	config->Write(_("wxWebServerDefaultDoc"), m_DefaultFile->GetValue());
   	config->Write(_("wxWebServerCgiExt"), m_CgiFileExt->GetValue());
   	config->Write(_("wxWebServerCgiHandler"), m_CgiHandler->GetPath());

	config->Write(_("wxWebServerAssumedContentTyper"), m_AssumedContentType->GetValue() );
	
	if( m_DoHeaders->GetValue() )
		config->Write(_("wxWebServerProcessNoHeadersResponse"), 1);
	else
		config->Write(_("wxWebServerProcessNoHeadersResponse"), 0);

	delete config;

	if( m_server  ) {
		m_server->Destroy();
		delete m_server;
	}
/*
// SEE THE COMMENT IN the Init() before uncommenting this!
#ifdef __WINDOWS__
// return the original value
    HKEY hKey;
    LONG lRet;
	
	try {
		if( dwConsoleSize != 0 ) {
		    lRet = RegOpenKeyEx( HKEY_CURRENT_USER,  TEXT("Console\\%SystemRoot%_system32_cmd.exe"), 0, KEY_QUERY_VALUE, &hKey );
			if( lRet == ERROR_SUCCESS ) {
				DWORD Type = REG_DWORD;
				DWORD Size = sizeof(dwConsoleSize);
				lRet = RegSetValueExW( hKey, TEXT("WindowSize"),  NULL, REG_DWORD, (LPBYTE) &dwConsoleSize, Size );
				RegCloseKey( hKey );
			}
		}
	} catch (int e) {
		dwConsoleSize = 0;
	}
#endif
*/
}


/*!
 * Member initialisation
 */

void WebServerMain::Init()
{
////@begin WebServerMain member initialisation
    m_DocRoot = NULL;
    m_Port = NULL;
    m_DefaultFile = NULL;
    m_ServiceStatus = NULL;
    m_CgiFileExt = NULL;
    m_CgiHandler = NULL;
    m_DoHeaders = NULL;
    m_AssumedContentType = NULL;
    m_ButtonStart = NULL;
    m_ButtonStop = NULL;
    m_ButtonExit = NULL;
////@end WebServerMain member initialisation

	m_server = NULL;

/* 
// If you really hate that command prompt popup you may enable this code and one in the destuctor
// It will reduce the size of a shell window to make it less annoying. You will need to run this application
// as Administator or change permissions for the registry key in question
#ifdef __WINDOWS__
// for Windows, we will reduce size of the CMD.EXE window so it's not so noticable when running wxShell for CGI processing
    HKEY hKey;
    LONG lRet;
	
    dwConsoleSize = 0;
	try {
	    lRet = RegOpenKeyEx( HKEY_CURRENT_USER,  TEXT("Console\\%SystemRoot%_system32_cmd.exe"), 0, KEY_QUERY_VALUE, &hKey );
		if( lRet == ERROR_SUCCESS ) {
			DWORD Type = REG_DWORD;
			DWORD Size = sizeof(dwConsoleSize);
			DWORD NewValue = 0x00100010;
			lRet = RegQueryValueExW( hKey, TEXT("WindowSize"),  NULL, &Type, (LPBYTE) &dwConsoleSize, &Size );
			lRet = RegSetValueExW( hKey, TEXT("WindowSize"),  NULL, REG_DWORD, (LPBYTE) &NewValue, Size );
			RegCloseKey( hKey );
		}
	} catch (int e) {
		dwConsoleSize = 0;
	}
#endif
*/
}


/*!
 * Control creation for WebServerMain
 */

void WebServerMain::CreateControls()
{    
////@begin WebServerMain content construction
    WebServerMain* itemDialog1 = this;

    wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxVERTICAL);
    itemDialog1->SetSizer(itemBoxSizer2);

    wxNotebook* itemNotebook3 = new wxNotebook( itemDialog1, ID_NOTEBOOK1, wxDefaultPosition, wxDefaultSize, wxBK_DEFAULT|wxNO_BORDER );

    wxPanel* itemPanel4 = new wxPanel( itemNotebook3, ID_PANEL_SERVER, wxDefaultPosition, wxDefaultSize, wxNO_BORDER|wxTAB_TRAVERSAL );
    if (ShowToolTips())
        itemPanel4->SetToolTip(_("Basic server configuration and status"));
    wxBoxSizer* itemBoxSizer5 = new wxBoxSizer(wxVERTICAL);
    itemPanel4->SetSizer(itemBoxSizer5);

    wxFlexGridSizer* itemFlexGridSizer6 = new wxFlexGridSizer(4, 2, 5, 10);
    itemFlexGridSizer6->AddGrowableCol(1);
    itemBoxSizer5->Add(itemFlexGridSizer6, 1, wxGROW|wxALL, 5);
    wxStaticText* itemStaticText7 = new wxStaticText( itemPanel4, wxID_STATIC, _("Document Root:"), wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT );
    itemFlexGridSizer6->Add(itemStaticText7, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_DocRoot = new wxDirPickerCtrl( itemPanel4, ID_DOCROOT, _T(""), _T(""), wxDefaultPosition, wxDefaultSize, wxDIRP_DEFAULT_STYLE|wxDIRP_DIR_MUST_EXIST );
    if (ShowToolTips())
        m_DocRoot->SetToolTip(_("Where html and other files are located"));
    itemFlexGridSizer6->Add(m_DocRoot, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText9 = new wxStaticText( itemPanel4, wxID_STATIC, _("Listen on port:"), wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT );
    itemFlexGridSizer6->Add(itemStaticText9, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_Port = new wxTextCtrl( itemPanel4, ID_PORT, _("8080"), wxDefaultPosition, wxSize(80, -1), 0 );
    m_Port->SetMaxLength(5);
    if (ShowToolTips())
        m_Port->SetToolTip(_("Port to listen on"));
    itemFlexGridSizer6->Add(m_Port, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText11 = new wxStaticText( itemPanel4, wxID_STATIC, _("Default file:"), wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT );
    itemFlexGridSizer6->Add(itemStaticText11, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_DefaultFile = new wxTextCtrl( itemPanel4, ID_DEFAULTFILE, _("index.html"), wxDefaultPosition, wxSize(180, -1), 0 );
    if (ShowToolTips())
        m_DefaultFile->SetToolTip(_("File to serve if '/' request"));
    itemFlexGridSizer6->Add(m_DefaultFile, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText13 = new wxStaticText( itemPanel4, wxID_STATIC, _("Service status:"), wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT );
    itemFlexGridSizer6->Add(itemStaticText13, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_ServiceStatus = new wxStaticText( itemPanel4, wxID_STATIC, _("Stopped"), wxDefaultPosition, wxDefaultSize, 0 );
    if (ShowToolTips())
        m_ServiceStatus->SetToolTip(_("Current status for the service"));
    m_ServiceStatus->SetForegroundColour(wxColour(255, 0, 0));
    itemFlexGridSizer6->Add(m_ServiceStatus, 0, wxALIGN_LEFT|wxGROW|wxALL, 5);

    itemNotebook3->AddPage(itemPanel4, _("Server"));

    wxPanel* itemPanel15 = new wxPanel( itemNotebook3, ID_PANEL_ADVANCED, wxDefaultPosition, wxDefaultSize, wxNO_BORDER|wxTAB_TRAVERSAL );
    if (ShowToolTips())
        itemPanel15->SetToolTip(_("Basic server configuration and status info"));
    wxBoxSizer* itemBoxSizer16 = new wxBoxSizer(wxVERTICAL);
    itemPanel15->SetSizer(itemBoxSizer16);

    wxStaticText* itemStaticText17 = new wxStaticText( itemPanel15, wxID_STATIC, _("Configure support for Python, Perl, PHP or anything else using CGI interface."), wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT );
    itemBoxSizer16->Add(itemStaticText17, 0, wxGROW|wxALL, 5);

    wxFlexGridSizer* itemFlexGridSizer18 = new wxFlexGridSizer(2, 2, 0, 0);
    itemFlexGridSizer18->AddGrowableCol(1);
    itemBoxSizer16->Add(itemFlexGridSizer18, 1, wxGROW|wxALL, 5);
    wxStaticText* itemStaticText19 = new wxStaticText( itemPanel15, wxID_STATIC, _("File extension:"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer18->Add(itemStaticText19, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_CgiFileExt = new wxTextCtrl( itemPanel15, ID_CGIEXT, _(".py"), wxDefaultPosition, wxDefaultSize, 0 );
    if (ShowToolTips())
        m_CgiFileExt->SetToolTip(_("File extension like .py or .php"));
    itemFlexGridSizer18->Add(m_CgiFileExt, 0, wxALIGN_LEFT|wxALIGN_TOP|wxALL, 5);

    wxStaticText* itemStaticText21 = new wxStaticText( itemPanel15, wxID_STATIC, _("CGI Handler:"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer18->Add(itemStaticText21, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_CgiHandler = new wxFilePickerCtrl( itemPanel15, ID_CGIHANDLER, _T("set path to python (perl, php, ...)"), _T(""), _T(""), wxDefaultPosition, wxDefaultSize, wxFLP_DEFAULT_STYLE );
    if (ShowToolTips())
        m_CgiHandler->SetToolTip(_("CGI handler for the above files"));
    itemFlexGridSizer18->Add(m_CgiHandler, 0, wxGROW|wxGROW|wxALL, 5);

    wxBoxSizer* itemBoxSizer23 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer16->Add(itemBoxSizer23, 0, wxGROW|wxALL, 5);
    m_DoHeaders = new wxCheckBox( itemPanel15, ID_DOHEADERS, _("Return raw response as content-type"), wxDefaultPosition, wxDefaultSize, 0 );
    m_DoHeaders->SetValue(false);
    if (ShowToolTips())
        m_DoHeaders->SetToolTip(_("Force content-type headers.\nUse it for testing scripts not producing proper CGI response."));
    itemBoxSizer23->Add(m_DoHeaders, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_AssumedContentType = new wxTextCtrl( itemPanel15, ID_ASSUMEDCONTENTTYPE, _("text/plain"), wxDefaultPosition, wxDefaultSize, 0 );
    if (ShowToolTips())
        m_AssumedContentType->SetToolTip(_("Force content-type headers.\nUse it for testing scripts not producing proper CGI response."));
    itemBoxSizer23->Add(m_AssumedContentType, 1, wxGROW|wxALL, 5);

    wxStaticText* itemStaticText26 = new wxStaticText( itemPanel15, wxID_STATIC, _("WARNING: read docs before changing this!"), wxDefaultPosition, wxDefaultSize, 0 );
    itemStaticText26->SetForegroundColour(wxColour(153, 0, 0));
    itemBoxSizer16->Add(itemStaticText26, 0, wxGROW|wxALL, 5);

    itemNotebook3->AddPage(itemPanel15, _("Advanced"));

    itemBoxSizer2->Add(itemNotebook3, 0, wxGROW|wxALL, 5);

    wxBoxSizer* itemBoxSizer27 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer2->Add(itemBoxSizer27, 0, wxGROW|wxALL, 5);

    wxButton* itemButton28 = new wxButton( itemDialog1, ID_BUTTON_ABOUT, _("About"), wxDefaultPosition, wxDefaultSize, 0 );
    if (ShowToolTips())
        itemButton28->SetToolTip(_("About this application"));
    itemBoxSizer27->Add(itemButton28, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxBoxSizer* itemBoxSizer29 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer27->Add(itemBoxSizer29, 1, wxGROW|wxALL, 0);

    wxStaticText* itemStaticText30 = new wxStaticText( itemDialog1, wxID_STATIC, _("                                "), wxDefaultPosition, wxDefaultSize, 0 );
    itemStaticText30->Enable(false);
    itemBoxSizer29->Add(itemStaticText30, 1, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_ButtonStart = new wxButton( itemDialog1, ID_BUTTON_START, _("Start"), wxDefaultPosition, wxDefaultSize, 0 );
    if (ShowToolTips())
        m_ButtonStart->SetToolTip(_("Start service with above parameters"));
    itemBoxSizer29->Add(m_ButtonStart, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_ButtonStop = new wxButton( itemDialog1, ID_BUTTON_STOP, _("Stop"), wxDefaultPosition, wxDefaultSize, 0 );
    if (ShowToolTips())
        m_ButtonStop->SetToolTip(_("Stop service, needed to change same parameters"));
    m_ButtonStop->Enable(false);
    itemBoxSizer29->Add(m_ButtonStop, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_ButtonExit = new wxButton( itemDialog1, ID_BUTTON_EXIT, _("Exit"), wxDefaultPosition, wxDefaultSize, 0 );
    if (ShowToolTips())
        m_ButtonExit->SetToolTip(_("Stop service and exit application"));
    itemBoxSizer29->Add(m_ButtonExit, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

////@end WebServerMain content construction

	wxConfig *config = new wxConfig(_("wxWebServer"));
	wxString str;
	long lval;
	if ( config->Read(_("wxWebServerDocRoot"), &str) ) {
		m_DocRoot->SetPath( str );
	}
	else {
		// set default value
	    wxStandardPaths* l_paths = new wxStandardPaths();
#ifdef __WINDOWS__		
	    wxString l_docFolder = l_paths->GetDocumentsDir() + _("\\htdocs");
#else
	    wxString l_docFolder = l_paths->GetDocumentsDir() + _("/htdocs");
#endif
		m_DocRoot->SetPath( l_docFolder );
	    delete l_paths;
	}	

	if ( config->Read(_("wxWebServerPort"), &str) ) {
		m_Port->SetValue( str );
	}

	if ( config->Read(_("wxWebServerDefaultDoc"), &str) ) {
		m_DefaultFile->SetValue( str );
	}

	if ( config->Read(_("wxWebServerCgiExt"), &str) ) {
		m_CgiFileExt->SetValue( str );
	}

	if ( config->Read(_("wxWebServerCgiHandler"), &str) ) {
		m_CgiHandler->SetPath( str );
	}

	if ( config->Read(_("wxWebServerAssumedContentTyper"), &str) ) {
		m_AssumedContentType->SetValue( str );
	}

	if ( config->Read(_("wxWebServerProcessNoHeadersResponse"), &lval) ) {
		m_DoHeaders->SetValue( lval != 0 );
	}

	delete config;

}


/*!
 * Should we show tooltips?
 */

bool WebServerMain::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap WebServerMain::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin WebServerMain bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end WebServerMain bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon WebServerMain::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin WebServerMain icon retrieval
    wxUnusedVar(name);
    if (name == _T("wxWebServer.xpm"))
    {
        wxIcon icon(wxWebServer_xpm);
        return icon;
    }
    return wxNullIcon;
////@end WebServerMain icon retrieval
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON_ABOUT
 */

void WebServerMain::OnButtonAboutClick( wxCommandEvent& event )
{
////@begin wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON_ABOUT in WebServerMain.
    // Before editing this code, remove the block markers.
    AboutDialog* window = new AboutDialog(this, ID_ABOUTDIALOG, _("About wxWebServer"));
    int returnValue = window->ShowModal();
    window->Destroy();
////@end wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON_ABOUT in WebServerMain. 
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON_START
 */

void WebServerMain::OnButtonStartClick( wxCommandEvent& event )
{
////@begin wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON_START in WebServerMain.
    // Before editing this code, remove the block markers.
    event.Skip();
////@end wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON_START in WebServerMain.

	// Create the address and socket
	wxIPV4address addr;
	addr.Service(m_Port->GetValue());
	m_server = new wxSocketServer(addr);

	// Check if started OK?
	if (!m_server->Ok())
	{
        wxMessageBox( _("Could not start listener at port ")+m_Port->GetValue(), _("Unable to start service"), wxICON_ERROR);
        delete m_server;
	}
	else
	{
        // Setup the event handler and subscribe to connection events
        m_server->SetEventHandler(*this, ID_SERVER);
        m_server->SetNotify(wxSOCKET_CONNECTION_FLAG);
        m_server->Notify(true);

        m_DocRoot->Disable();
        m_Port->Disable();
        m_ServiceStatus->SetLabel(_("Service Running"));
        m_ServiceStatus->SetForegroundColour(wxColour(0, 108, 54));
        m_ButtonStart->Disable();
        m_ButtonStop->Enable();
	}
 
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON_STOP
 */

void WebServerMain::OnButtonStopClick( wxCommandEvent& event )
{
////@begin wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON_STOP in WebServerMain.
    // Before editing this code, remove the block markers.
    event.Skip();
////@end wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON_STOP in WebServerMain. 

    if( m_server->Destroy() )
    {
        m_DocRoot->Enable();
        m_Port->Enable();
        m_ServiceStatus->SetLabel(_("Service Stopped"));
        m_ServiceStatus->SetForegroundColour(wxColour(255, 0, 0));
        m_ButtonStart->Enable();
        m_ButtonStop->Disable();
        delete m_server;
		m_server = NULL;
    } 
    else 
    {
        wxMessageBox( _("Unable to stop service!"), _("Unknown error"), wxICON_ERROR);
        m_ServiceStatus->SetLabel(_("Unknown!"));
        m_ServiceStatus->SetForegroundColour(wxColour(255, 0, 0));
    }
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON_EXIT
 */

void WebServerMain::OnButtonExitClick( wxCommandEvent& event )
{
////@begin wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON_EXIT in WebServerMain.
    // Before editing this code, remove the block markers.
    EndModal(ID_BUTTON_EXIT);
////@end wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON_EXIT in WebServerMain. 
}


/*!
 * OnServerEvent
 */
void WebServerMain::OnServerEvent(wxSocketEvent& WXUNUSED(event))
{
	// Accept the connection
	wxSocketBase* sock = m_server->Accept(true);
	sock->SetTimeout(2);
    sock->SetFlags(wxSOCKET_NONE);

	// no headers handling
	wxString nhCT = wxEmptyString; 
	if( m_DoHeaders->GetValue() )
		nhCT = m_AssumedContentType->GetValue();
	
	wxString* reqLog = new wxString();
	wxString* resLog = new wxString();

	// Instantiate new handler for this request
	new CRequestHandler(sock, m_DocRoot->GetPath(), m_DefaultFile->GetValue(), 
							m_CgiFileExt->GetValue(), m_CgiHandler->GetPath(), nhCT,
							reqLog, resLog );

	if( reqLog->Length() > MAX_LOG )
		reqLog->Truncate( MAX_LOG );
	if( resLog->Length() > MAX_LOG )
		resLog->Truncate( MAX_LOG );

	delete reqLog;
	delete resLog;
}

