/////////////////////////////////////////////////////////////////////////////
//
// wxWebServer - a simple web server using wxWidgets framework
//
// File:        webservermain.h
// 
// Author:      Fedja Stevanovic
// 
// Copyright:   Copyright (C) 2007 Fedja Stevanovic (tutubibi.net)
//
// Licence:
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////////////

#ifndef _WEBSERVERMAIN_H_
#define _WEBSERVERMAIN_H_

#if defined(__GNUG__) && !defined(NO_GCC_PRAGMA)
#pragma interface "webservermain.h"
#endif

/*!
 * Includes
 */

////@begin includes
#include "wx/notebook.h"
#include "wx/filepicker.h"
////@end includes

#include "wx/stdpaths.h"
#include "wx/socket.h"

/*!
 * Forward declarations
 */

////@begin forward declarations
class wxDirPickerCtrl;
class wxFilePickerCtrl;
////@end forward declarations

/*!
 * Control identifiers
 */

////@begin control identifiers
#define ID_WXWEBSERVER 10000
#define ID_NOTEBOOK1 10003
#define ID_PANEL_SERVER 10007
#define ID_DOCROOT 10001
#define ID_PORT 10002
#define ID_DEFAULTFILE 10012
#define ID_PANEL_ADVANCED 10013
#define ID_CGIEXT 10014
#define ID_CGIHANDLER 10015
#define ID_DOHEADERS 10022
#define ID_ASSUMEDCONTENTTYPE 10018
#define ID_BUTTON_ABOUT 10008
#define ID_BUTTON_START 10004
#define ID_BUTTON_STOP 10005
#define ID_BUTTON_EXIT 10006
#define SYMBOL_WEBSERVERMAIN_STYLE wxCAPTION|wxSYSTEM_MENU|wxCLOSE_BOX|wxMINIMIZE_BOX|wxSIMPLE_BORDER
#define SYMBOL_WEBSERVERMAIN_TITLE _("wxWebServer")
#define SYMBOL_WEBSERVERMAIN_IDNAME ID_WXWEBSERVER
#define SYMBOL_WEBSERVERMAIN_SIZE wxDefaultSize
#define SYMBOL_WEBSERVERMAIN_POSITION wxDefaultPosition
////@end control identifiers

#define ID_SERVER 11000
#define ID_SOCKET 11001

/*!
 * WebServerMain class declaration
 */

class WebServerMain: public wxDialog
{    
    DECLARE_DYNAMIC_CLASS( WebServerMain )
    DECLARE_EVENT_TABLE()

public:
    /// Constructors
    WebServerMain();
    WebServerMain( wxWindow* parent, wxWindowID id = SYMBOL_WEBSERVERMAIN_IDNAME, const wxString& caption = SYMBOL_WEBSERVERMAIN_TITLE, const wxPoint& pos = SYMBOL_WEBSERVERMAIN_POSITION, const wxSize& size = SYMBOL_WEBSERVERMAIN_SIZE, long style = SYMBOL_WEBSERVERMAIN_STYLE );

    /// Creation
    bool Create( wxWindow* parent, wxWindowID id = SYMBOL_WEBSERVERMAIN_IDNAME, const wxString& caption = SYMBOL_WEBSERVERMAIN_TITLE, const wxPoint& pos = SYMBOL_WEBSERVERMAIN_POSITION, const wxSize& size = SYMBOL_WEBSERVERMAIN_SIZE, long style = SYMBOL_WEBSERVERMAIN_STYLE );

    /// Destructor
    ~WebServerMain();

    /// Initialises member variables
    void Init();

    /// Creates the controls and sizers
    void CreateControls();

////@begin WebServerMain event handler declarations

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON_ABOUT
    void OnButtonAboutClick( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON_START
    void OnButtonStartClick( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON_STOP
    void OnButtonStopClick( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON_EXIT
    void OnButtonExitClick( wxCommandEvent& event );

////@end WebServerMain event handler declarations

////@begin WebServerMain member function declarations

    /// Retrieves bitmap resources
    wxBitmap GetBitmapResource( const wxString& name );

    /// Retrieves icon resources
    wxIcon GetIconResource( const wxString& name );
////@end WebServerMain member function declarations

    /// Should we show tooltips?
    static bool ShowToolTips();

////@begin WebServerMain member variables
    wxDirPickerCtrl* m_DocRoot;
    wxTextCtrl* m_Port;
    wxTextCtrl* m_DefaultFile;
    wxStaticText* m_ServiceStatus;
    wxTextCtrl* m_CgiFileExt;
    wxFilePickerCtrl* m_CgiHandler;
    wxCheckBox* m_DoHeaders;
    wxTextCtrl* m_AssumedContentType;
    wxButton* m_ButtonStart;
    wxButton* m_ButtonStop;
    wxButton* m_ButtonExit;
////@end WebServerMain member variables

    wxSocketServer* m_server;
    void OnServerEvent(wxSocketEvent& event);

#ifdef __WINDOWS__
	DWORD dwConsoleSize;
#endif

};

#endif
    // _WEBSERVERMAIN_H_
